"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var path = require("path");
var fs = require("fs");
var Http = require("http");
var express = require("express");
var Utils_1 = require("./Utils");
var winston = require("winston");
var helmet = require("helmet");
var morgan = require("morgan");
var cookieParser = require("cookie-parser");
var bodyParser = require("body-parser");
var serveFavicon = require("serve-favicon");
var Session = require("express-session");
var FileStore = require("session-file-store");
var md5 = require("md5");
var User_1 = require("./User");
var glob = require("glob");
var util = require("util");
var pgPromise = require("pg-promise");
require("winston-daily-rotate-file");
var HttpApplication = (function () {
    function HttpApplication(appName, rootFolder, options) {
        this.getAcceptedLangugeHeader = function (req) {
            var data = req.headers["accept-language"];
            if (data) {
                var lst = data.split(",");
                if (lst.length !== 0) {
                    return lst[0];
                }
                else {
                    data;
                }
            }
            else {
                return null;
            }
        };
        var me = this, viewPath = path.join(__dirname, "..", "resources", "views", "error");
        me.isTestMode = false;
        me.httpErrorViews = {};
        me.setHttpErrorView(404, path.join(viewPath, "404.pug"));
        me.setHttpErrorView(500, path.join(viewPath, "500.pug"));
        me.options = options || {};
        me.rootFolder = rootFolder;
        me.appName = appName;
        me.expressApp = express();
        me.expressApp.set(HttpApplication.KEY_APPLICATION_NAME, appName);
        me.expressApp.set(HttpApplication.KEY_ROOT_FOLDER, rootFolder);
        me.normalizeOptions();
        me.createRuntimeFolders();
        me.createLogger();
    }
    HttpApplication.prototype.enterTestMode = function () {
        var me = this;
        me.isTestMode = true;
    };
    HttpApplication.prototype.setHttpErrorView = function (httpStatus, viewFullPath) {
        var me = this;
        if (fs.existsSync(viewFullPath)) {
            me.httpErrorViews[httpStatus] = viewFullPath;
        }
        else {
            throw new Error("Could not resolve " + viewFullPath);
        }
    };
    HttpApplication.prototype.createLogger = function () {
        var me = this, logFolder = Utils_1.Utils.ensureFolder(path.join(me.varFolder, "log"));
        me.logger = new winston.Logger({ exitOnError: false });
        me.logger.configure({
            transports: [
                new (winston.transports.File)({
                    name: "error-log",
                    level: "error",
                    maxsize: me.options.maxLogSize,
                    filename: path.join(logFolder, "error.log")
                }),
                new (winston.transports.DailyRotateFile)({
                    name: "daily-log",
                    maxsize: me.options.maxLogSize,
                    filename: path.join(logFolder, "application-"),
                    datePattern: "yyyy-MM.log",
                    level: me.options.appLogLevel
                })
            ]
        });
        me.logFolder = logFolder;
        me.expressApp.set("_logger", me.logger);
        me.logger.info("%s loggers initialized with [%s] level", me.appName, me.options.appLogLevel.toUpperCase());
    };
    HttpApplication.prototype.normalizeOptions = function () {
        var me = this;
        me.options.port = me.options.port || 8090;
        me.options.address = me.options.address || "127.0.0.1";
        me.options.appLogLevel = me.options.appLogLevel || (me.isDevel() ? "debug" : me.getEnv("loglevel", "info"));
        me.options.maxLogSize = me.options.maxLogSize || (10 * (1024 * 1024));
        me.options.runtimeFolder = me.options.runtimeFolder || path.join(me.rootFolder, "var");
    };
    HttpApplication.prototype.isDevel = function () {
        var me = this;
        return me.getEnv("env", "development") !== "production";
    };
    HttpApplication.prototype.getEnv = function (key, defValue) {
        var me = this, key = (me.appName + "_" + key);
        return (process.env[key.toLowerCase()] || process.env[key.toUpperCase()] || defValue);
    };
    HttpApplication.prototype.configureHelmetSecurity = function (options) {
        var me = this;
        me.logger.info("Configuring Helmet");
        me.expressApp.use(helmet(options));
        me.expressApp.use(helmet.hidePoweredBy({ setTo: "PHP 4.2.0" }));
        return true;
    };
    HttpApplication.prototype.configureHttpLogger = function () {
        var me = this, loggerName = me.createHttpLogger(), httpLogger = winston.loggers.get(loggerName), jsonCombined = function (tokens, req, res) {
            return JSON.stringify({
                remote_user: tokens["remote-user"](req, res) || "anonymus",
                remote_addr: tokens["remote-addr"](req),
                method: tokens["method"](req, res),
                status: tokens["status"](req, res),
                timestamp: tokens["date"](req, res, "iso"),
                url: tokens["url"](req, res),
                request_headers: req.headers,
                response_headers: res._headers,
                response_time: tokens["response-time"](req, res)
            });
        };
        var args = [
            jsonCombined,
            {
                stream: {
                    write: function (message) {
                        httpLogger.info(message.trim());
                    }
                }
            }
        ];
        me.expressApp.use(morgan.apply(morgan, args));
        return true;
    };
    HttpApplication.prototype.createHttpLogger = function () {
        var me = this, loggerName = "http-logger";
        winston.loggers.add(loggerName, {
            transports: [
                new (winston.transports.DailyRotateFile)({
                    name: "http-log",
                    filename: path.join(me.logFolder, "access-"),
                    datePattern: "yyyy-MM.log",
                    level: me.options.appLogLevel,
                    json: false,
                    maxsize: me.options.maxLogSize,
                    formatter: function (options) {
                        return options["message"];
                    }
                })
            ]
        });
        return loggerName;
    };
    HttpApplication.prototype.configurePostgreSQLDatabase = function (host, database, username, password, port, options) {
        var me = this, constr = util.format("postgres://%s:%s@%s:%s/%s", username, password, host, port || 5432, database);
        me.pgp = pgPromise(options || {});
        me.database = me.pgp(constr);
    };
    HttpApplication.prototype.configureContentParsers = function () {
        var me = this;
        me.expressApp.use(cookieParser());
        me.expressApp.use(bodyParser.json());
        me.expressApp.use(bodyParser.urlencoded({ extended: false }));
        return true;
    };
    HttpApplication.prototype.configureFilebasedSessionContainer = function () {
        var me = this, sessionFolder = Utils_1.Utils.ensureFolder(path.join(me.varFolder, ".session")), fileStoreOptions = {
            path: sessionFolder,
            useAsync: true,
            reapInterval: 5000,
            maxAge: 10000,
        }, sessionOptions = {
            secret: md5(me.rootFolder),
            name: md5(me.rootFolder),
            cookie: {}
        };
        if (!me.isDevel()) {
            me.expressApp.set("trust proxy", 1);
            if (!me.isTestMode) {
                sessionOptions.cookie["secure"] = true;
            }
        }
        sessionOptions.store = new (FileStore(Session))(fileStoreOptions);
        sessionOptions.resave = true;
        sessionOptions.saveUninitialized = true;
        me.expressApp.use(Session(sessionOptions));
        return true;
    };
    HttpApplication.prototype.configureSessionContainer = function () {
        var me = this;
        return me.configureFilebasedSessionContainer();
    };
    HttpApplication.prototype.getCurrentUser = function (req) {
        return req.session[HttpApplication.KEY_AUTH_USER] || null;
    };
    HttpApplication.prototype.setCurrentUser = function (req, user) {
        req.session[HttpApplication.KEY_AUTH_USER] = user;
    };
    HttpApplication.prototype.getAvailableLocales = function (req) {
        return req.app.get(HttpApplication.KEY_AVAILABLE_LOCALES) || [];
    };
    HttpApplication.prototype.getCurrentLocale = function (req) {
        var me = this;
        return req.session[HttpApplication.KEY_LOCALE] || me.getAcceptedLangugeHeader(req) || "en";
    };
    HttpApplication.prototype.getContentType = function (req) {
        return req.headers["content-type"] || null;
    };
    HttpApplication.prototype.configureAuthenticatedUser = function () {
        var me = this;
        me.expressApp.use(function (req, res, next) {
            if (!me.getCurrentUser(req)) {
                req.session[HttpApplication.KEY_AUTH_USER] = new User_1.AnonymousUser();
            }
            next();
        });
        return true;
    };
    HttpApplication.prototype.configureFileBasedTranslationProvider = function (folders) {
        var me = this, tokens = {
            "${rootFolder}": me.rootFolder,
            "${thisFolder}": null
        }, file = null, db = {};
        var parseValue = function (value, sourceDir) {
            tokens["${thisFolder}"] = sourceDir;
            for (var k in tokens) {
                if (value.indexOf(k) === 0) {
                    debugger;
                    file = fs.realpathSync(value.replace(k, tokens[k]));
                    value = fs.readFileSync(file).toLocaleString();
                }
            }
            return value;
        };
        var procesData = function (data, file) {
            for (var msg in data) {
                for (var loc in data[msg]) {
                    if (!db[loc]) {
                        db[loc] = {};
                    }
                    db[loc][msg] = parseValue(data[msg][loc], path.dirname(file));
                }
            }
        };
        folders.forEach(function (item) {
            glob.sync(item).forEach(function (file) {
                try {
                    procesData(JSON.parse(fs.readFileSync(file).toLocaleString()), file);
                }
                catch (e) {
                    me.logger.error(e.message);
                }
            });
        });
        me.expressApp.set(HttpApplication.KEY_I18N, db);
    };
    HttpApplication.prototype.translate = function (req, message) {
        var args = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            args[_i - 2] = arguments[_i];
        }
        return util.format.apply(util.format, [((req.app.get(HttpApplication.KEY_I18N) || {})[this.getCurrentLocale(req)] || {})[message] || message].concat(args));
    };
    HttpApplication.prototype.configureLocaleProvider = function (parameterName, defaultLocale, availableLocales) {
        var me = this;
        me.expressApp.use(function (req, res, next) {
            var accept_language = me.getAcceptedLangugeHeader(req);
            defaultLocale = defaultLocale || accept_language || "en";
            availableLocales = availableLocales || [defaultLocale];
            if (availableLocales.indexOf(defaultLocale) === -1) {
                availableLocales.push(defaultLocale);
            }
            var get_param = req.query[parameterName] || null;
            var post_param = req.body[parameterName] || null;
            var sess_param = me.getCurrentLocale(req) || null;
            var value = get_param || post_param || sess_param || accept_language || defaultLocale;
            value = availableLocales.indexOf(value) !== -1 ? value : defaultLocale;
            req.session[HttpApplication.KEY_LOCALE] = value;
            if (!me.getAvailableLocales(req)) {
                req.app.set(HttpApplication.KEY_AVAILABLE_LOCALES, availableLocales);
            }
            next();
        });
        return true;
    };
    HttpApplication.prototype.configureStaticFolderProvider = function (folders) {
        var me = this;
        Object.keys(folders).forEach(function (mapping) {
            me.logger.info("Serving %s as %s", folders[mapping], mapping);
            if (mapping === "/") {
                me.expressApp.use(express.static(folders[mapping], {
                    fallthrough: true
                }));
            }
            else {
                me.expressApp.use("/" + mapping, express.static(folders[mapping], {
                    fallthrough: false
                }));
            }
        });
        return true;
    };
    HttpApplication.prototype.configureFaciconProvider = function (faviconPath) {
        var me = this;
        me.expressApp.use(serveFavicon(faviconPath));
    };
    HttpApplication.prototype.configurePugTemplatesProvider = function (pugOptions) {
        var me = this;
        me.expressApp.set("view engine", "pug");
        me.expressApp.set("view options", pugOptions || {});
    };
    HttpApplication.prototype.configureApplicationInternal = function () {
        var me = this, ok = false;
        if (me.configureHelmetSecurity()) {
            if (me.configureHttpLogger()) {
                if (me.configureContentParsers()) {
                    if (me.configureSessionContainer()) {
                        if (me.configureAuthenticatedUser()) {
                            ok = true;
                        }
                    }
                }
            }
        }
        return ok;
    };
    HttpApplication.prototype.configure404StatusProvider = function () {
        var me = this;
        me.expressApp.use(function (req, res, next) {
            var err = new Error(req.originalUrl + " Not Found!");
            err.status = 404;
            next(err);
        });
    };
    HttpApplication.prototype.handleFinalJSONError = function (err, req, res, next) {
        var me = this;
        res.json({
            error: true,
            message: err.mesage,
            status: err.status,
            stack: me.isDevel() ? err.stack : null
        });
    };
    HttpApplication.prototype.handleFinalHTMLError = function (err, req, res, next) {
        var me = this, errorViewFile = me.httpErrorViews[err.status];
        fs.exists(errorViewFile, function (exists) {
            if (exists) {
                res.render(errorViewFile, {
                    appName: me.appName,
                    err: err,
                    locale: me.getCurrentLocale(req) || "en"
                });
            }
            else {
                res.send("Error: " + err.status + "\nMessage: " + err.message + "\nStack: " + (me.isDevel() ? err.stack : ""));
            }
        });
    };
    HttpApplication.prototype.configureFinalErrorProvider = function () {
        var me = this;
        me.expressApp.use(function (err, req, res, next) {
            me.logger.error("Error", {
                status: err.status,
                message: err.message,
                url: req.url,
                stack: err.stack,
                req_headers: req.headers,
            });
            var contentType = me.getContentType(req) || false;
            err.status = err.status || 500;
            err.message = me.isDevel() ? err.message : "Error with status " + err.status + " has been set";
            err.stack = me.isDevel() ? err.stack : null;
            res.setHeader("Content-Type", contentType || "");
            res.status(err.status);
            res.setHeader("Cache-Control", "private, no-cache, no-store, max-age=0, proxy-revalidate, s-maxage=0");
            res.setHeader("Expires", "Sat, 12 Oct 1991 05:00:00 GMT");
            res.setHeader("Pragma", "no-cache");
            res.setHeader("Vary", "*");
            if (contentType && contentType.indexOf("application/json") !== -1) {
                me.handleFinalJSONError.apply(me, arguments);
            }
            else {
                me.handleFinalHTMLError.apply(me, arguments);
            }
        });
    };
    HttpApplication.prototype.createRuntimeFolders = function () {
        var me = this;
        me.varFolder = Utils_1.Utils.ensureFolder(me.options.runtimeFolder);
        me.expressApp.set(HttpApplication.KEY_RUNTIME_FOLDER, me.varFolder);
    };
    HttpApplication.prototype.start = function (done) {
        var me = this;
        if (!me.running) {
            me.httpServer = Http.createServer(me.expressApp);
            me.httpServer.on("error", function () {
                me.onError.apply(me, arguments);
            });
            me.httpServer.on("listening", function () {
                var addr = me.httpServer.address();
                me.logger.info("%s is ready for connections on address: %s port: %s", me.appName, addr.address, addr.port);
                if (me.onStart.apply(me, arguments) === true) {
                    me.running = true;
                    if (done) {
                        done();
                    }
                }
            });
            if (me.configureApplicationInternal()) {
                me.configureApplication();
                me.configure404StatusProvider();
                me.configureFinalErrorProvider();
                me.httpServer.listen(me.options.port, me.options.address);
            }
        }
        else {
            if (done) {
                done();
            }
        }
    };
    HttpApplication.prototype.stop = function (done) {
        var me = this;
        if (me.running) {
            me.httpServer.close(function () {
                me.onStop.apply(me, arguments);
                me.logger.info("%s is shutdown.", me.appName);
                me.logger.close();
                winston.loggers.close("http-logger");
                me.running = false;
                if (done) {
                    done();
                }
            });
        }
        else {
            if (done) {
                done();
            }
        }
    };
    HttpApplication.prototype.onError = function (error) {
        var me = this, msg = "";
        if (error.syscall !== "listen") {
            throw error;
        }
        switch (error.code) {
            case "EACCES":
                msg = me.options.port + " requires elevated privileges!";
                me.logger.error(msg);
                console.error(msg);
                process.exit(1);
                break;
            case "EADDRINUSE":
                msg = me.options.port + " is already in use!";
                me.logger.error(msg);
                console.error(msg);
                process.exit(1);
                break;
            default:
                throw error;
        }
    };
    ;
    HttpApplication.prototype.onStart = function () {
        return true;
    };
    HttpApplication.prototype.onStop = function () {
    };
    return HttpApplication;
}());
HttpApplication.KEY_RUNTIME_FOLDER = "_runtime_folder";
HttpApplication.KEY_ROOT_FOLDER = "_root_folder";
HttpApplication.KEY_LOG_FOLDER = "_log_folder";
HttpApplication.KEY_APPLICATION_NAME = "_application_name";
HttpApplication.KEY_AUTH_USER = "_auth_user";
HttpApplication.KEY_AVAILABLE_LOCALES = "_available_locales";
HttpApplication.KEY_LOCALE = "_locale";
HttpApplication.KEY_I18N = "_i18n";
exports.HttpApplication = HttpApplication;
//# sourceMappingURL=HttpApplication.js.map